#!/usr/bin/env node
"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));

// src/cli.ts
var import_meow = __toESM(require("meow"), 1);
var import_chalk = __toESM(require("chalk"), 1);

// src/html.ts
var import_node_path = __toESM(require("path"), 1);
var import_node_fs = __toESM(require("fs"), 1);
var import_promises = __toESM(require("fs/promises"), 1);
var import_meta = {};
var htmlTemplate = (diff) => `<html>
<head>
  <title>GraphQL Schema Diff</title>
  <meta charset="utf-8" />
  <link rel="stylesheet" href="css/hljs.min.css">
  <link rel="stylesheet" href="css/diff2html.min.css">
  <script src="js/highlight.min.js"></script>
  <script src="js/graphql.min.js"></script>
  <script src="js/diff2html.min.js"></script>
  <script src="js/diff2html-ui-base.min.js "></script>
</head>
<body>
  <div id="diff"></div>
  <script>
    document.addEventListener('DOMContentLoaded', () => {
      const diffString = \`${diff}\`;
      const targetElement = document.getElementById('diff');
      const configuration = {
        matching: 'lines',
        outputFormat: 'side-by-side',
        highlight: true,
        synchronisedScroll: false,
        drawFileList: false,
        rawTemplates: {
          'tag-file-renamed': '',
        }
      };
      const diff2htmlUi = new Diff2HtmlUI(targetElement, diffString, configuration, hljs);
      diff2htmlUi.draw();
    });
  </script>
</body>
</html>`;
async function createHtmlOutput(diff, options = {}) {
  const { outputDirectory = "schemaDiff" } = options;
  const adjustedDiff = diff.replace(/(---\s.*)\sremoved/, "$1").replace(/(\+\+\+\s.*)\sadded/, "$1").replace("No newline at end of file", "");
  if (!import_node_fs.default.existsSync(import_node_path.default.join(outputDirectory, "js"))) {
    import_node_fs.default.mkdirSync(import_node_path.default.join(outputDirectory, "js"), { recursive: true });
  }
  if (!import_node_fs.default.existsSync(import_node_path.default.join(outputDirectory, "css"))) {
    import_node_fs.default.mkdirSync(import_node_path.default.join(outputDirectory, "css"), { recursive: true });
  }
  const diff2HtmlPath = new URL(
    ".",
    import_meta.resolve("diff2html/package.json")
  ).pathname;
  const highlightJsPath = new URL(
    ".",
    import_meta.resolve("@highlightjs/cdn-assets/package.json")
  ).pathname;
  await Promise.all([
    import_promises.default.copyFile(
      import_node_path.default.join(diff2HtmlPath, "bundles/js/diff2html.min.js"),
      import_node_path.default.join(outputDirectory, "js/diff2html.min.js")
    ),
    import_promises.default.copyFile(
      import_node_path.default.join(diff2HtmlPath, "bundles/css/diff2html.min.css"),
      import_node_path.default.join(outputDirectory, "css/diff2html.min.css")
    ),
    import_promises.default.copyFile(
      import_node_path.default.join(highlightJsPath, "styles/default.min.css"),
      import_node_path.default.join(outputDirectory, "css/hljs.min.css")
    ),
    import_promises.default.copyFile(
      import_node_path.default.join(highlightJsPath, "highlight.min.js"),
      import_node_path.default.join(outputDirectory, "js/highlight.min.js")
    ),
    import_promises.default.copyFile(
      import_node_path.default.join(highlightJsPath, "languages/graphql.min.js"),
      import_node_path.default.join(outputDirectory, "js/graphql.min.js")
    )
  ]);
  const diff2htmlUiBase = (await import_promises.default.readFile(
    import_node_path.default.join(diff2HtmlPath, "bundles/js/diff2html-ui-base.min.js")
  )).toString().replace('void 0!==t?t:"plaintext"', '"graphql"');
  const htmlOutput = htmlTemplate(adjustedDiff);
  await Promise.all([
    import_promises.default.writeFile(import_node_path.default.join(outputDirectory, "index.html"), htmlOutput),
    import_promises.default.writeFile(
      import_node_path.default.join(outputDirectory, "js/diff2html-ui-base.min.js"),
      diff2htmlUiBase
    )
  ]);
}

// src/diff.ts
var import_graphql = require("graphql");
var import_lexicographicSortSchema = require("graphql/utilities/lexicographicSortSchema.js");
var import_disparity = __toESM(require("disparity"), 1);
var import_load = require("@graphql-tools/load");
var import_url_loader = require("@graphql-tools/url-loader");
var import_json_file_loader = require("@graphql-tools/json-file-loader");
var import_utils = require("@graphql-tools/utils");
var import_graphql_file_loader = require("@graphql-tools/graphql-file-loader");
var import_node_fetch = __toESM(require("node-fetch"), 1);
async function getDiff(leftSchemaLocation2, rightSchemaLocation2, options = {}) {
  const getSchemaOptions = (customHeaders) => ({
    headers: { ...options.headers, ...customHeaders },
    skipGraphQLImport: false,
    descriptions: false,
    customFetch: import_node_fetch.default,
    inputValueDeprecation: options.inputValueDeprecation
  });
  const leftSchemaOptions = getSchemaOptions(options.leftSchema?.headers);
  const rightSchemaOptions = getSchemaOptions(options.rightSchema?.headers);
  let [leftSchema, rightSchema] = await Promise.all([
    (0, import_load.loadSchema)(leftSchemaLocation2, {
      loaders: [new import_url_loader.UrlLoader(), new import_json_file_loader.JsonFileLoader(), new import_graphql_file_loader.GraphQLFileLoader()],
      ...leftSchemaOptions
    }),
    (0, import_load.loadSchema)(rightSchemaLocation2, {
      loaders: [new import_url_loader.UrlLoader(), new import_json_file_loader.JsonFileLoader(), new import_graphql_file_loader.GraphQLFileLoader()],
      ...rightSchemaOptions
    })
  ]);
  if (!leftSchema || !rightSchema) {
    throw new Error("Schemas not defined");
  }
  if (options.sortSchema) {
    [leftSchema, rightSchema] = [
      (0, import_lexicographicSortSchema.lexicographicSortSchema)(leftSchema),
      (0, import_lexicographicSortSchema.lexicographicSortSchema)(rightSchema)
    ];
  }
  const [leftSchemaSDL, rightSchemaSDL] = [
    (0, import_utils.printSchemaWithDirectives)(leftSchema),
    (0, import_utils.printSchemaWithDirectives)(rightSchema)
  ];
  if (leftSchemaSDL === rightSchemaSDL) {
    return;
  }
  const diff = import_disparity.default.unified(leftSchemaSDL, rightSchemaSDL, {
    paths: [leftSchemaLocation2, rightSchemaLocation2]
  });
  const diffNoColor = import_disparity.default.unifiedNoColor(leftSchemaSDL, rightSchemaSDL, {
    paths: [leftSchemaLocation2, rightSchemaLocation2]
  });
  const dangerousChanges = (0, import_graphql.findDangerousChanges)(leftSchema, rightSchema);
  const breakingChanges = (0, import_graphql.findBreakingChanges)(leftSchema, rightSchema);
  return {
    diff,
    diffNoColor,
    dangerousChanges,
    breakingChanges
  };
}

// src/cli.ts
var cli = (0, import_meow.default)(
  `
  Usage
    $ graphql-schema-diff <leftSchemaLocation> <rightSchemaLocation>

  Options
    --fail-on-dangerous-changes  Exit with error on dangerous changes
    --fail-on-breaking-changes 	 Exit with error on breaking changes
    --fail-on-all-changes 	 Exit with error on all changes
    --create-html-output 	 Creates an HTML file containing the diff
    --html-output-directory 	 Directory where the HTML file should be stored (Default: './schemaDiff')
    --header, -H 		 Header to send to all remote schema sources
    --left-schema-header 	 Header to send to left remote schema source
    --right-schema-header 	 Header to send to right remote schema source
    --sort-schema, -s 		 Sort schemas prior to diffing
    --input-value-deprecation 	 Include deprecated input value fields when loading from URL

  Examples
    $ graphql-schema-diff https://example.com/graphql schema.graphql
    $ graphql-schema-diff https://example.com/graphql schema.graphql -H 'Authorization: Bearer 123'
`,
  {
    flags: {
      failOnDangerousChanges: {
        type: "boolean"
      },
      failOnBreakingChanges: {
        type: "boolean"
      },
      failOnAllChanges: {
        type: "boolean"
      },
      // Deprecated: chalk.supportsColor should be used instead
      useColors: {
        type: "boolean"
      },
      createHtmlOutput: {
        type: "boolean"
      },
      htmlOutputDirectory: {
        type: "string",
        default: "schemaDiff"
      },
      header: {
        type: "string",
        alias: "H"
      },
      leftSchemaHeader: {
        type: "string"
      },
      rightSchemaHeader: {
        type: "string"
      },
      sortSchema: {
        type: "boolean",
        alias: "s"
      },
      help: {
        alias: "h"
      },
      version: {
        alias: "v"
      },
      inputValueDeprecation: {
        type: "boolean"
      }
    }
  }
);
function parseHeaders(headerInput) {
  let headers;
  const parseHeader = (header2) => header2.split(":").map((val) => val.trim());
  if (!headerInput) return;
  if (Array.isArray(headerInput)) {
    headers = headerInput.map(parseHeader);
  } else {
    headers = [parseHeader(headerInput)];
  }
  return headers.reduce(
    (result, [key, value]) => ({
      ...result,
      ...key && value && { [key]: value }
    }),
    {}
  );
}
var [leftSchemaLocation, rightSchemaLocation] = cli.input;
var {
  header,
  leftSchemaHeader,
  rightSchemaHeader
} = cli.flags;
if (!leftSchemaLocation || !rightSchemaLocation) {
  console.error(
    import_chalk.default.red("ERROR: Schema locations missing!\n\n"),
    import_chalk.default.gray(
      "Usage\n  $ graphql-schema-diff <leftSchemaLocation> <rightSchemaLocation>"
    )
  );
  process.exit(1);
}
getDiff(leftSchemaLocation, rightSchemaLocation, {
  headers: parseHeaders(header),
  leftSchema: {
    headers: parseHeaders(leftSchemaHeader)
  },
  rightSchema: {
    headers: parseHeaders(rightSchemaHeader)
  },
  sortSchema: cli.flags.sortSchema,
  inputValueDeprecation: cli.flags.inputValueDeprecation
}).then(async (result) => {
  if (result === void 0) {
    console.warn(import_chalk.default.green("\u2714 No changes"));
    return;
  }
  const hasBreakingChanges = result.breakingChanges.length !== 0;
  const hasDangerousChanges = result.dangerousChanges.length !== 0;
  if (cli.flags.useColors || import_chalk.default.supportsColor) {
    console.log(result.diff);
  } else {
    console.log(result.diffNoColor);
  }
  if (hasDangerousChanges) {
    console.warn(import_chalk.default.yellow.bold.underline("Dangerous changes"));
    for (const change of result.dangerousChanges) {
      console.warn(import_chalk.default.yellow("  \u26A0 " + change.description));
    }
  }
  if (hasDangerousChanges && hasBreakingChanges) {
    console.warn();
  }
  if (hasBreakingChanges) {
    console.warn(import_chalk.default.red.bold.underline("BREAKING CHANGES"));
    for (const change of result.breakingChanges) {
      console.warn(import_chalk.default.red("  \u2716 " + change.description));
    }
  }
  if (cli.flags.createHtmlOutput) {
    await createHtmlOutput(result.diffNoColor, {
      outputDirectory: cli.flags.htmlOutputDirectory
    });
  }
  if (hasDangerousChanges && cli.flags.failOnDangerousChanges || hasBreakingChanges && cli.flags.failOnBreakingChanges || cli.flags.failOnAllChanges) {
    process.exit(1);
    return;
  }
}).catch((err) => {
  console.error(import_chalk.default.red(`
ERROR: ${err.message}`));
  process.exit(1);
});
//# sourceMappingURL=cli.cjs.map